==> synaptic_0.16.orig.tar.gz: synaptic-0.16.orig/common/rpackagecache.cc <==
/* rpackagecache.cc - package cache wrapper
 * 
 * Copyright (c) 2000, 2001 Conectiva S/A 
 * 
 * Author: Alfredo K. Kojima <kojima@conectiva.com.br>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "config.h"

#include "rpackagecache.h"

#include "i18n.h"

#include <apt-pkg/error.h>
#include <apt-pkg/sourcelist.h>
#include <apt-pkg/pkgcachegen.h>
#include <apt-pkg/configuration.h>
#ifdef HAVE_RPM
# include <apt-pkg/systemfactory.h>
#else
# include <apt-pkg/pkgsystem.h>
# include <apt-pkg/policy.h>
#endif



bool RPackageCache::open(OpProgress &progress)
{
#ifdef HAVE_RPM
    _lock = new pkgRpmLock(true);
#else
    _system->Lock();
#endif

    if (_error->PendingError())
	return false;
    
    // Read the source list
    pkgSourceList list;
    if (!list.ReadMainList())
       return _error->Error(_("The list of sources could not be read."));
    
#ifdef HAVE_RPM
    _system->makeStatusCache(list, progress);
    progress.Done();
#else
    pkgMakeStatusCache(list, progress);
#endif

    if (_error->PendingError())
	return _error->Error(_("The package lists or status file could not be parsed or opened."));

    // Open the cache file
    FileFd File(_config->FindFile("Dir::Cache::pkgcache"),FileFd::ReadOnly);
    if (_error->PendingError())
	return false;

    _map = new MMap(File,MMap::Public | MMap::ReadOnly);
    if (_error->PendingError())
	return false;
    
    // Create the dependency cache
#ifdef HAVE_RPM
    _dcache = new pkgDepCache(*_map, progress);
#else
    _cache = new pkgCache(_map);
    if (_error->PendingError())
       return false;
    
    // The policy engine
    _policy = new pkgPolicy(_cache);
    if (_error->PendingError() == true)
       return false;
    if (ReadPinFile(*_policy) == false)
       return false;
    
    _dcache = new pkgDepCache(_cache, _policy);
    
    _dcache->Init(&progress);
#endif

    progress.Done();
    if (_error->PendingError())
	return false;

    // Check that the system is OK
    if (_dcache->DelCount() != 0 || _dcache->InstCount() != 0)
	return _error->Error("Internal Error, non-zero counts");

    _locked = true;

    return true;
}



bool RPackageCache::reset(OpProgress &progress)
{
    delete _dcache;
    delete _map;

    releaseLock();

#ifdef HAVE_RPM
    if (_lock) {
	delete _lock;
	_lock = 0;
    }
#endif
    return open(progress);
}



bool RPackageCache::lock()
{
    if (_locked)
       return true;

#ifdef HAVE_RPM
    if (_lock) {
	_lock->GetLock(true);
    }
#else    
    _system->Lock();
#endif
    _locked = true;
}


void RPackageCache::releaseLock()
{
    if (!_locked)
	return;

#ifdef HAVE_RPM
    if (_lock) {
	_lock->Close();
    }
#else    
    _system->UnLock();
#endif
    _locked = false;
}

==> synaptic-0.48.1.tar.gz: synaptic-0.48.1/common/rpackagecache.cc <==
/* rpackagecache.cc - package cache wrapper
 * 
 * Copyright (c) 2000-2003 Conectiva S/A 
 * 
 * Author: Alfredo K. Kojima <kojima@conectiva.com.br>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "config.h"

#include "rpackagecache.h"

#include "i18n.h"

#include<assert.h>
#include <algorithm>
#include <apt-pkg/error.h>
#include <apt-pkg/sourcelist.h>
#include <apt-pkg/pkgcachegen.h>
#include <apt-pkg/configuration.h>
#include <apt-pkg/pkgsystem.h>
#include <apt-pkg/policy.h>


bool RPackageCache::open(OpProgress &progress)
{
    _system->Lock();

    if (_error->PendingError())
	return false;
    
    // Read the source list
    //pkgSourceList list;
    assert(_list!=NULL);
    if (!_list->ReadMainList())
       return _error->Error(_("The list of sources could not be read.\n\
Go to the repository dialog to correct the problem."));

    pkgMakeStatusCache(*_list, progress);

    if (_error->PendingError())
	return _error->Error(_("The package lists or status file could not be parsed or opened."));

    // Open the cache file
    FileFd File(_config->FindFile("Dir::Cache::pkgcache"),FileFd::ReadOnly);
    if (_error->PendingError())
	return false;

    _map = new MMap(File,MMap::Public | MMap::ReadOnly);
    if (_error->PendingError())
	return false;
    
    // Create the dependency cache
    _cache = new pkgCache(_map);
    if (_error->PendingError())
       return false;
    
    // The policy engine
    if(_policy != NULL)
      delete _policy;
    _policy = new pkgPolicy(_cache);
    if (_error->PendingError() == true)
       return false;
    if (ReadPinFile(*_policy) == false)
       return false;
    
    _dcache = new pkgDepCache(_cache, _policy);
    
    _dcache->Init(&progress);

    //progress.Done();
    if (_error->PendingError())
	return false;

    // Check that the system is OK
    if (_dcache->DelCount() != 0 || _dcache->InstCount() != 0)
	return _error->Error(_("Internal Error, non-zero counts"));

    _locked = true;

    return true;
}



bool RPackageCache::reset(OpProgress &progress)
{
    delete _dcache;
    delete _map;

    releaseLock();

    return open(progress);
}
    

vector<string> RPackageCache::getPolicyArchives() 
{
    //std::cout << "RPackageCache::getPolicyComponents() " << std::endl;

    vector<string> archives;
    for (pkgCache::PkgFileIterator F = _cache->FileBegin(); F.end() == false;
	 F++)
	{
	    pkgIndexFile *Indx;
	    _list->FindIndex(F,Indx);
	    _system->FindIndex(F,Indx);
	    
	    if ( !F.RelStr().empty() ) {
// 		printf("Archive: %s, Origin: %s, Component: %s\n", 
// 		       F.Archive(), F.Origin(), F.Component());
		if(F.Archive() != NULL) {
		    if(find(archives.begin(),archives.end(),F.Archive())
		       == archives.end()) {
			archives.push_back(F.Archive());
		    }
		}
	    }
	}
    return archives;
}


bool RPackageCache::lock()
{
    if (_locked)
       return true;

    _system->Lock();
    _locked = true;

    //FIXME: should depend on the result of _system->lock()
    return true;
}


void RPackageCache::releaseLock()
{
    if (!_locked)
	return;

    _system->UnLock();
    _locked = false;
}
